#include "torture.h"
#include "cmap.h"

#define COLORS 256
#define CHANNELS 3
#define pj_copy_bytes(s,d,n) memcpy((d), (s), (n))

/*****************************************************************************
 * Default color map.
 *	There's nothing special about this other than an attempt to have adjacent
 *	colors with a good visual contrast to aid visual verification of patterns.
 ****************************************************************************/

static Rgb3 pj_default_cmap[COLORS] = {
	{0x00, 0x00, 0x00}, 	/*	 0 */
	{0x00, 0x00, 0xff}, 	/*	 1 */
	{0x00, 0xff, 0x00}, 	/*	 2 */
	{0xff, 0x00, 0x00}, 	/*	 3 */
	{0xba, 0x52, 0xce}, 	/*	 4 */
	{0x00, 0x1c, 0xcf}, 	/*	 5 */
	{0xe6, 0xe6, 0x2a}, 	/*	 6 */
	{0xa1, 0x00, 0x33}, 	/*	 7 */
	{0x56, 0xe5, 0xe4}, 	/*	 8 */
	{0xf4, 0x73, 0xc0}, 	/*	 9 */
	{0x00, 0x00, 0xa4}, 	/*	10 */
	{0x63, 0xd0, 0x3c}, 	/*	11 */
	{0xef, 0xb7, 0x69}, 	/*	12 */
	{0x91, 0x95, 0x4f}, 	/*	13 */
	{0xa2, 0x56, 0x9b}, 	/*	14 */
	{0xd1, 0xcb, 0xdb}, 	/*	15 */
	{0x5e, 0xcc, 0x85}, 	/*	16 */
	{0x56, 0xa4, 0xde}, 	/*	17 */
	{0xa5, 0x50, 0xd1}, 	/*	18 */
	{0xbc, 0x45, 0x84}, 	/*	19 */
	{0x3c, 0x2e, 0xc1}, 	/*	20 */
	{0xe6, 0x9d, 0x2b}, 	/*	21 */
	{0xc3, 0x65, 0x9d}, 	/*	22 */
	{0x5b, 0x5b, 0xb5}, 	/*	23 */
	{0xeb, 0xe8, 0x84}, 	/*	24 */
	{0xf4, 0x75, 0x44}, 	/*	25 */
	{0x32, 0xd3, 0xdc}, 	/*	26 */
	{0xd1, 0x54, 0x44}, 	/*	27 */
	{0xd7, 0x8a, 0xd5}, 	/*	28 */
	{0xda, 0xff, 0x85}, 	/*	29 */
	{0xee, 0x3c, 0xa5}, 	/*	30 */
	{0x96, 0xeb, 0x8e}, 	/*	31 */
	{0x86, 0xe2, 0xcc}, 	/*	32 */
	{0xed, 0x62, 0x78}, 	/*	33 */
	{0x68, 0x6a, 0x2f}, 	/*	34 */
	{0xdf, 0xe9, 0x67}, 	/*	35 */
	{0x82, 0x97, 0x7a}, 	/*	36 */
	{0x6a, 0x72, 0xc9}, 	/*	37 */
	{0x7a, 0x4c, 0xad}, 	/*	38 */
	{0x36, 0xdd, 0xc1}, 	/*	39 */
	{0xaf, 0x00, 0xce}, 	/*	40 */
	{0x5d, 0xbf, 0xab}, 	/*	41 */
	{0xb7, 0xb3, 0x70}, 	/*	42 */
	{0x29, 0x4a, 0xcf}, 	/*	43 */
	{0x6f, 0xd0, 0x62}, 	/*	44 */
	{0x72, 0x61, 0xbc}, 	/*	45 */
	{0x73, 0x00, 0x5a}, 	/*	46 */
	{0x83, 0xa6, 0x72}, 	/*	47 */
	{0x5d, 0xad, 0xb9}, 	/*	48 */
	{0x38, 0x25, 0xd8}, 	/*	49 */
	{0xbe, 0x8f, 0x3a}, 	/*	50 */
	{0x5d, 0x87, 0x58}, 	/*	51 */
	{0xea, 0xac, 0xd9}, 	/*	52 */
	{0x6c, 0xaa, 0x48}, 	/*	53 */
	{0xb1, 0x43, 0x8c}, 	/*	54 */
	{0x1c, 0x00, 0xbf}, 	/*	55 */
	{0x72, 0xde, 0x78}, 	/*	56 */
	{0x9b, 0x3f, 0x60}, 	/*	57 */
	{0xa5, 0xca, 0x4c}, 	/*	58 */
	{0xad, 0x42, 0xe0}, 	/*	59 */
	{0xbc, 0xd9, 0xa1}, 	/*	60 */
	{0xe2, 0x6f, 0xeb}, 	/*	61 */
	{0x70, 0x6c, 0x00}, 	/*	62 */
	{0x82, 0x4a, 0x59}, 	/*	63 */
	{0xec, 0x86, 0x3c}, 	/*	64 */
	{0x59, 0x80, 0x97}, 	/*	65 */
	{0x95, 0xdf, 0xd1}, 	/*	66 */
	{0xc9, 0xda, 0xbf}, 	/*	67 */
	{0x52, 0x6e, 0x8f}, 	/*	68 */
	{0x9b, 0x9b, 0x9b}, 	/*	69 */
	{0xdd, 0x82, 0x45}, 	/*	70 */
	{0x40, 0xb4, 0x00}, 	/*	71 */
	{0x2f, 0x9b, 0xbf}, 	/*	72 */
	{0x99, 0x66, 0x65}, 	/*	73 */
	{0x8a, 0x8a, 0xbf}, 	/*	74 */
	{0x4c, 0x2d, 0xb6}, 	/*	75 */
	{0x7b, 0xff, 0x00}, 	/*	76 */
	{0x75, 0x2f, 0xd4}, 	/*	77 */
	{0xa7, 0xc2, 0x92}, 	/*	78 */
	{0xdc, 0xac, 0x53}, 	/*	79 */
	{0xc0, 0xb7, 0xa2}, 	/*	80 */
	{0xa8, 0x89, 0x2e}, 	/*	81 */
	{0xb0, 0x85, 0x39}, 	/*	82 */
	{0x3d, 0xcf, 0x7f}, 	/*	83 */
	{0x3d, 0xbd, 0xea}, 	/*	84 */
	{0xc0, 0x42, 0x4a}, 	/*	85 */
	{0x4c, 0xa2, 0x46}, 	/*	86 */
	{0x91, 0x8b, 0xb6}, 	/*	87 */
	{0xd3, 0x30, 0x38}, 	/*	88 */
	{0x52, 0xbf, 0xbb}, 	/*	89 */
	{0xa3, 0x54, 0xdd}, 	/*	90 */
	{0x00, 0x47, 0xc9}, 	/*	91 */
	{0x39, 0xc3, 0x55}, 	/*	92 */
	{0xff, 0xd5, 0x58}, 	/*	93 */
	{0x40, 0x8e, 0xce}, 	/*	94 */
	{0x29, 0xc0, 0x29}, 	/*	95 */
	{0x6e, 0xa8, 0x99}, 	/*	96 */
	{0x9c, 0x9c, 0xfc}, 	/*	97 */
	{0xe6, 0xb8, 0xc6}, 	/*	98 */
	{0x51, 0xa5, 0x77}, 	/*	99 */
	{0xba, 0x52, 0xce}, 	/* 100 */
	{0x00, 0x67, 0x47}, 	/* 101 */
	{0x7a, 0xa5, 0xe4}, 	/* 102 */
	{0x7e, 0x53, 0xd8}, 	/* 103 */
	{0x6c, 0xed, 0xb8}, 	/* 104 */
	{0xd7, 0x29, 0x3a}, 	/* 105 */
	{0x9d, 0xa7, 0x3a}, 	/* 106 */
	{0xe5, 0x8c, 0x48}, 	/* 107 */
	{0xda, 0xe6, 0xbd}, 	/* 108 */
	{0xb5, 0x9f, 0xa0}, 	/* 109 */
	{0xa6, 0x49, 0xea}, 	/* 110 */
	{0xe2, 0xa3, 0x85}, 	/* 111 */
	{0x4b, 0x72, 0xa5}, 	/* 112 */
	{0x64, 0x52, 0xa9}, 	/* 113 */
	{0x3a, 0xca, 0x75}, 	/* 114 */
	{0xac, 0x58, 0xe8}, 	/* 115 */
	{0x64, 0x64, 0x64}, 	/* 116 */
	{0x89, 0x9f, 0x5a}, 	/* 117 */
	{0xbc, 0xaa, 0x33}, 	/* 118 */
	{0x71, 0xb2, 0x54}, 	/* 119 */
	{0xe6, 0xa4, 0x5b}, 	/* 120 */
	{0xb8, 0x28, 0x7b}, 	/* 121 */
	{0x2c, 0x57, 0xcd}, 	/* 122 */
	{0x38, 0x67, 0x36}, 	/* 123 */
	{0xbc, 0xa2, 0x2a}, 	/* 124 */
	{0xc9, 0xc9, 0xc9}, 	/* 125 */
	{0x58, 0x7b, 0xa2}, 	/* 126 */
	{0xea, 0x3c, 0x90}, 	/* 127 */
	{0x74, 0x00, 0x53}, 	/* 128 */
	{0x73, 0xec, 0x3e}, 	/* 129 */
	{0x83, 0x5d, 0xe4}, 	/* 130 */
	{0x47, 0x84, 0x65}, 	/* 131 */
	{0xd3, 0x75, 0x5f}, 	/* 132 */
	{0x58, 0x2b, 0x81}, 	/* 133 */
	{0xa0, 0x4c, 0x71}, 	/* 134 */
	{0x81, 0x8b, 0x4b}, 	/* 135 */
	{0x47, 0xae, 0x58}, 	/* 136 */
	{0xb7, 0xc1, 0x59}, 	/* 137 */
	{0xc5, 0x42, 0x52}, 	/* 138 */
	{0x43, 0x60, 0xc3}, 	/* 139 */
	{0x5c, 0x46, 0x7d}, 	/* 140 */
	{0xe8, 0xe1, 0xb6}, 	/* 141 */
	{0x69, 0xff, 0x34}, 	/* 142 */
	{0x87, 0x4e, 0xb1}, 	/* 143 */
	{0x56, 0x3e, 0x81}, 	/* 144 */
	{0x7c, 0xc2, 0x67}, 	/* 145 */
	{0x85, 0x28, 0x38}, 	/* 146 */
	{0x9a, 0x98, 0x75}, 	/* 147 */
	{0xca, 0xc4, 0xae}, 	/* 148 */
	{0x89, 0x60, 0x67}, 	/* 149 */
	{0x5a, 0x43, 0xab}, 	/* 150 */
	{0xb8, 0x00, 0x77}, 	/* 151 */
	{0x6c, 0xac, 0x9a}, 	/* 152 */
	{0x00, 0x00, 0xa4}, 	/* 153 */
	{0x7f, 0x33, 0xd5}, 	/* 154 */
	{0x4f, 0x8b, 0xa0}, 	/* 155 */
	{0x8c, 0x00, 0xd8}, 	/* 156 */
	{0xb3, 0xeb, 0x90}, 	/* 157 */
	{0x47, 0x3f, 0xc4}, 	/* 158 */
	{0xbd, 0x00, 0x00}, 	/* 159 */
	{0x6e, 0xd2, 0xc3}, 	/* 160 */
	{0xa3, 0xff, 0x1c}, 	/* 161 */
	{0x3d, 0x3d, 0xb5}, 	/* 162 */
	{0xef, 0xb7, 0x69}, 	/* 163 */
	{0x4d, 0x3e, 0xbb}, 	/* 164 */
	{0x7d, 0x39, 0xa3}, 	/* 165 */
	{0x30, 0x47, 0xae}, 	/* 166 */
	{0x58, 0xdb, 0x45}, 	/* 167 */
	{0x81, 0x00, 0x37}, 	/* 168 */
	{0x93, 0xdf, 0xd2}, 	/* 169 */
	{0xb5, 0x4b, 0x85}, 	/* 170 */
	{0x8e, 0xc1, 0x57}, 	/* 171 */
	{0x38, 0x6e, 0x6a}, 	/* 172 */
	{0xef, 0xec, 0x50}, 	/* 173 */
	{0xb2, 0x67, 0x00}, 	/* 174 */
	{0x00, 0xb2, 0xff}, 	/* 175 */
	{0x61, 0xec, 0x46}, 	/* 176 */
	{0x42, 0x31, 0xca}, 	/* 177 */
	{0x7e, 0x3f, 0x58}, 	/* 178 */
	{0x4f, 0x97, 0x2c}, 	/* 179 */
	{0x9b, 0x33, 0x32}, 	/* 180 */
	{0x36, 0x9c, 0x71}, 	/* 181 */
	{0xe6, 0xb4, 0xee}, 	/* 182 */
	{0x70, 0xc4, 0xbb}, 	/* 183 */
	{0x73, 0x37, 0x74}, 	/* 184 */
	{0xc9, 0x7d, 0x80}, 	/* 185 */
	{0xa4, 0xb9, 0x65}, 	/* 186 */
	{0xef, 0x4f, 0x97}, 	/* 187 */
	{0xff, 0xff, 0x2d}, 	/* 188 */
	{0xbb, 0x2b, 0x92}, 	/* 189 */
	{0x5e, 0x2e, 0x89}, 	/* 190 */
	{0xda, 0x8f, 0xcd}, 	/* 191 */
	{0x34, 0x43, 0xd0}, 	/* 192 */
	{0x78, 0xeb, 0xde}, 	/* 193 */
	{0x7b, 0xa9, 0xcf}, 	/* 194 */
	{0xbb, 0x7d, 0x5d}, 	/* 195 */
	{0x59, 0xe5, 0xd9}, 	/* 196 */
	{0x54, 0x56, 0x57}, 	/* 197 */
	{0x49, 0xe0, 0x69}, 	/* 198 */
	{0xbb, 0x54, 0x6f}, 	/* 199 */
	{0x37, 0xc6, 0x8e}, 	/* 200 */
	{0x99, 0xd9, 0xc5}, 	/* 201 */
	{0x56, 0xda, 0x74}, 	/* 202 */
	{0x96, 0x38, 0xe3}, 	/* 203 */
	{0xc9, 0xeb, 0x00}, 	/* 204 */
	{0x49, 0x5a, 0xa5}, 	/* 205 */
	{0x31, 0xde, 0xda}, 	/* 206 */
	{0x6c, 0x99, 0x67}, 	/* 207 */
	{0xbc, 0xba, 0x2d}, 	/* 208 */
	{0xcc, 0xef, 0xb1}, 	/* 209 */
	{0x4b, 0x92, 0xe7}, 	/* 210 */
	{0xcc, 0x95, 0xc5}, 	/* 211 */
	{0x3c, 0x35, 0xe3}, 	/* 212 */
	{0xe0, 0x9d, 0xe8}, 	/* 213 */
	{0xc7, 0x7e, 0x4c}, 	/* 214 */
	{0x28, 0x00, 0xbb}, 	/* 215 */
	{0x5c, 0xad, 0x5a}, 	/* 216 */
	{0xaf, 0x80, 0x53}, 	/* 217 */
	{0xc4, 0x69, 0xbe}, 	/* 218 */
	{0x29, 0x53, 0x53}, 	/* 219 */
	{0xaf, 0x86, 0x44}, 	/* 220 */
	{0xb2, 0xb8, 0x84}, 	/* 221 */
	{0xe5, 0x9b, 0xbc}, 	/* 222 */
	{0x61, 0x66, 0xd4}, 	/* 223 */
	{0x46, 0x51, 0x73}, 	/* 224 */
	{0x8f, 0x28, 0x4e}, 	/* 225 */
	{0x4e, 0x89, 0x9b}, 	/* 226 */
	{0x3b, 0xae, 0x6f}, 	/* 227 */
	{0x65, 0xe3, 0x72}, 	/* 228 */
	{0xec, 0xa4, 0x34}, 	/* 229 */
	{0xc4, 0x3d, 0xe4}, 	/* 230 */
	{0xca, 0x35, 0x51}, 	/* 231 */
	{0x9c, 0x6d, 0x34}, 	/* 232 */
	{0xca, 0x90, 0x5a}, 	/* 233 */
	{0xd6, 0xd5, 0xa7}, 	/* 234 */
	{0x43, 0x95, 0x50}, 	/* 235 */
	{0x5a, 0xce, 0xb9}, 	/* 236 */
	{0x50, 0x92, 0xa6}, 	/* 237 */
	{0x76, 0x5f, 0x96}, 	/* 238 */
	{0xbb, 0x33, 0x68}, 	/* 239 */
	{0xc6, 0xda, 0xde}, 	/* 240 */
	{0x4b, 0x4b, 0xac}, 	/* 241 */
	{0xff, 0x36, 0x43}, 	/* 242 */
	{0x40, 0x33, 0x7a}, 	/* 243 */
	{0x9e, 0x28, 0x4e}, 	/* 244 */
	{0x39, 0x71, 0x34}, 	/* 245 */
	{0xc5, 0x90, 0x5d}, 	/* 246 */
	{0xe1, 0xe0, 0xb1}, 	/* 247 */
	{0x6f, 0xba, 0x43}, 	/* 248 */
	{0x7e, 0x00, 0x3c}, 	/* 249 */
	{0x00, 0x78, 0xc1}, 	/* 250 */
	{0xd4, 0x3a, 0x27}, 	/* 251 */
	{0xe6, 0xe6, 0x2a}, 	/* 252 */
	{0x52, 0xa5, 0x55}, 	/* 253 */
	{0xff, 0x7b, 0x00}, 	/* 254 */
	{0x95, 0x33, 0xc4}, 	/* 255 */
	};

static void cycle_cmap(UBYTE cmap[COLORS][CHANNELS])
/*****************************************************************************
 *
 ****************************************************************************/
{
UBYTE	color[CHANNELS];

	pj_copy_bytes(&cmap[0][0], color, sizeof(color));
	pj_copy_bytes(&cmap[1][0],&cmap[0][0], (COLORS-1)*CHANNELS);
	pj_copy_bytes(color, &cmap[COLORS-1][0], sizeof(color));
}

static void ramp_cmap(UBYTE cmap[COLORS][CHANNELS], int channel)
/*****************************************************************************
 *
 ****************************************************************************/
{
int i;

	memset(cmap, 0, COLORS*CHANNELS);
	for (i=0;i<256;i++)
		cmap[i][channel] = i;
}

void set_default_colors(Raster *r)
/*****************************************************************************
 *
 ****************************************************************************/
{
	log_start("Calling wait_vsync() then set_colors() to set default test palette...\n");
	r->lib->wait_vsync(r);
	r->lib->set_colors(r, 0, COLORS, pj_default_cmap);
	log_end("...default color palette set.\n\n");
}

void test_colors(Raster *r)
/*****************************************************************************
 *
 ****************************************************************************/
{
Rastlib 	*rlib = r->lib;
short		counter;
Vsyncfunc_t wait_vsync;
Colorfunc_t set_colors;
UBYTE		cmap[COLORS][CHANNELS];
int 		i,j;

	wait_vsync = rlib->wait_vsync;
	set_colors = rlib->set_colors;

	log_progress("Testing wait_vsync()...\n");
	counter = 30;
	while (--counter)
		wait_vsync(r);
	log_progress("...wait_vsync() testing complete.\n\n");

	log_start("Testing set_colors()...\n");

	for (j=0; j<CHANNELS; j++)
		{
		ramp_cmap(cmap, j);
		for (i=0; i<COLORS; i+=2)
			{
			wait_vsync(r);
			set_colors(r, 0, COLORS, cmap);
			cycle_cmap(cmap);
			}
		}

	log_end("...testing of set_colors() complete.\n\n");

	set_default_colors(r);

}

